/*
bhnetwork.cpp
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
    
*/
#include <errno.h>
#include "pos.h"
//#include "cmfengine.h"
#include "bhnetwork.h"

#ifdef _DEBUGGERY 
//extern int DebugLevel=3;
#endif

gint toActive(gpointer blah)
{
	CNetwork *net=(CNetwork *) blah;
	
	return net->Timeout();
}
void sdActive(gpointer blah, int d, GdkInputCondition c)
{
	CNetwork *net=(CNetwork *) blah;
	net->Activity(d, c);
}
void csdActive(gpointer blah, int d, GdkInputCondition c)
{
	CNetwork *net=(CNetwork *) blah;
	net->Ctrl_Activity(d, c);
}

gint CNetwork::Timeout()
{
	if(bResend){
		retries++;
		if(retries>=max_retries){
			// failure...
			gtk_timeout_remove(timer_tag);
         timeout_signal.emit();
			return 0;
		}
		if(bBackoff){
			timer_value+=(int)((float) timer_value * backoff_value);
			gtk_timeout_remove(timer_tag);
		}
		ReSend();
		return 1;
	}
	
	return 0;
}

void CNetwork::Ctrl_Activity(int d, GdkInputCondition c)
{
	int status;
	
	if(d!=ctrl_sd) return;	
	
	ctrlmutex.lock();
	
	if(c==GDK_INPUT_READ){
		status=recvfrom(ctrl_sd, &ctrlhdr, sizeof(ctrlhdr), 0, NULL, &size);
      ctrlmsg_signal.emit(size, &ctrlhdr);
	}
	ctrlmutex.unlock();
}

void CNetwork::Activity(int d, GdkInputCondition c)
{
	int 		status;
	
	if(d!=sd) return;
	
	netmutex.lock();
	
	if(bTimeout)
		gtk_timeout_remove(timer_tag);
		
	switch(c){
	case GDK_INPUT_READ:
		status=recvfrom(sd, &rxhdr, sizeof(rxhdr), 0, NULL, &size);
		if(status < 0) break;
		// server says stop sending this message
		if(rxhdr.hdr.msgID > 0) break;
		
		if(rxhdr.hdr.type==BHPOS_TYPE_SYSTEM && rxhdr.hdr.cmd==BHMSG_SYSTEM_CONNECTION && 
			(rxhdr.hdr.subcmd==BHMSG_SYSTEM_CONNECT || rxhdr.hdr.subcmd==BHMSG_SYSTEM_RECONNECT)){
				memcpy(&txhdr, &rxhdr, sizeof(BhHdr));
		}
      msg_signal.emit(size, &rxhdr);
		break;
		
	case GDK_INPUT_WRITE:
	case GDK_INPUT_EXCEPTION:
		break;
	}
	netmutex.unlock();
}

CNetwork::CNetwork()
{
	bTcp=false;
	memset(&posserv, 0, sizeof(posserv));
	memset(&ctrlserv, 0, sizeof(posserv));
	memset(macaddr, 0, sizeof(macaddr));
	bHaveServerAddress=false;
	bHaveCtrlAddress=false;
	bBackoff=true;
	max_retries=10;
	retries=0;
	backoff_value=0.3f;
	bResend=true;
	bTimeout=true;
	timer_tag=0;
	timer_value=2000;
}

CNetwork::CNetwork(Gtk::Main *m)
{
	bTcp=false;
	memset(&posserv, 0, sizeof(posserv));
	memset(&ctrlserv, 0, sizeof(posserv));
   strcpy(devname, "");	
	memset(macaddr, 0, sizeof(macaddr));
	bHaveServerAddress=false;
	bHaveCtrlAddress=false;
   bHaveDevice=false;
	bBackoff=true;
	max_retries=10;
	retries=0;
	backoff_value=0.3f;
	bResend=true;
	bTimeout=true;
	timer_tag=0;
	timer_value=2000;
}

CNetwork::~CNetwork()
{
	Close();
}

int CNetwork::SetServer(int t, char *a, unsigned short p)
{
	posserv.sin_family=t;
	posserv.sin_addr.s_addr=inet_addr(a);
	posserv.sin_port=htons(p);

	bHaveServerAddress=true;
	return 0;
}

int CNetwork::SetCtrl(int t, char *a, unsigned short p)
{
	ctrlserv.sin_family=t;
	ctrlserv.sin_addr.s_addr=inet_addr(a);
	ctrlserv.sin_port=htons(p);

	bHaveCtrlAddress=true;
	return 0;
}

int CNetwork::SetDevice(char *dev)
{
   if(dev==NULL) return 1;
	strncpy(devname, dev, 16);
	bHaveDevice=true;
	return 0;
}
		
int CNetwork::Open()
{
	socklen_t    opt=1;
	int          ttl=2;
	struct ifreq ifr;
	
	memset(&sin, 0, sizeof(sin));
	sin.sin_family=AF_INET;
	sin.sin_addr.s_addr=htonl(INADDR_ANY);
		
	memset(&csin, 0, sizeof(csin));
	csin.sin_family=AF_INET;
	csin.sin_addr.s_addr=htonl(INADDR_ANY);
		
	
	sd=socket(AF_INET, SOCK_DGRAM, 0);
	if(sd < 0){
#ifdef _DEBUGGERY
		g_print("no socket creation");
#endif
		return -1;
	}
   if(bHaveDevice){
      strcpy(ifr.ifr_name, devname); 
   }
   else{
   	strncpy(ifr.ifr_name, "eth0", 5); 
   }

#if defined(FREEBSD) || defined(NETBSD) || defined(OPENBSD)
   if(ioctl(sd, LINUX_SIOCGIFHWADDR, &ifr) >= 0){ 
#else
	if(ioctl(sd, SIOCGIFHWADDR, &ifr) >= 0){ 
#endif
		macaddr[0]=ifr.ifr_addr.sa_data[0]; 
		macaddr[1]=ifr.ifr_addr.sa_data[1]; 
		macaddr[2]=ifr.ifr_addr.sa_data[2]; 
		macaddr[3]=ifr.ifr_addr.sa_data[3]; 
		macaddr[4]=ifr.ifr_addr.sa_data[4]; 
		macaddr[5]=ifr.ifr_addr.sa_data[5]; 
#ifdef _DEBUGGERY
		g_print("aquired mac address - %X:%X:%X:%X:%X:%X\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],macaddr[4],macaddr[5]);
#endif		
	}
   else{
      g_print("failed to aquired mac address\n");
   }
	
	if(setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(int))<0){

#ifdef _DEBUGGERY
		g_print("no udp socket reuse");
#endif
	}
	if(bind(sd, (struct sockaddr *) &sin, sizeof(sin)) < 0){

#ifdef _DEBUGGERY
		g_print("no socket bind");
#endif
		return -2;
	}
	
	if(!bHaveServerAddress){
#ifdef _DEBUGGERY
      g_message("No Server Address, setting to localhost");
#endif
		posserv.sin_family=AF_INET;
		posserv.sin_addr.s_addr=inet_addr("127.0.0.1");
		posserv.sin_port=htons(6672);
	}

	
	if(!bHaveCtrlAddress){
#ifdef _DEBUGGERY
	g_message("No Control Address, setting to local mcast");
#endif
		ctrlserv.sin_family=AF_INET;
		ctrlserv.sin_addr.s_addr=inet_addr("224.0.0.1");
		ctrlserv.sin_port=htons(6673);
	}

	ctrl_sd=socket(AF_INET, SOCK_DGRAM, 0);
	if(ctrl_sd < 0){

#ifdef _DEBUGGERY
		g_print("no ctrl socket creation\n");
#endif 
		return -3;
	}
		
	if(setsockopt(ctrl_sd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(int))<0){

#ifdef _DEBUGGERY
		g_print("no ctrl socket reuse\n");
#endif
	}
		
		
	if((ttl=bind(ctrl_sd, (struct sockaddr *) &csin, sizeof(csin))) < 0){

#ifdef _DEBUGGERY
	g_print("no ctrl socket bind - error:%d - %s\n", ttl, strerror(errno));
#endif

		return -4;
	}
	// ??? FIX ME ??? this is a two hopper, should really be an option
	ttl=2;
	if(setsockopt(ctrl_sd, IPPROTO_IP, IP_MULTICAST_TTL, &ttl,sizeof(ttl))<0){
#ifdef _DEBUGGERY
		g_print("no ttl set\n");
#endif
	}
	
	mreq.imr_multiaddr.s_addr = ctrlserv.sin_addr.s_addr;
	mreq.imr_interface.s_addr=htonl(INADDR_ANY);
	if((ttl=setsockopt(ctrl_sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq)))<0){

#ifdef _DEBUGGERY
		g_print("no multicast group join - error:%d - %s\n", ttl, strerror(errno));
#endif

		//return -5;
	}

#ifdef _DEBUGGERY
		g_print("sockets all opened correctly\n");
#endif
	
   //net.create(sd, Glib::IO_IN);
   //net.connect(sigc::slot(sdActive));
   //main->input.connect(sigc::ptr_fun(sdActive), sd, GDK_INPUT_READ);
	sd_tag=gdk_input_add(sd, GDK_INPUT_READ, &sdActive, this);
	csd_tag=gdk_input_add(ctrl_sd, GDK_INPUT_READ, &csdActive, this);

	return 0;
}

void CNetwork::Close()
{
	gdk_input_remove(sd_tag);
	gdk_input_remove(csd_tag);
	close(sd);
	close(ctrl_sd);
}

int CNetwork::GetMac(unsigned char *m)
{
   if(m){
      memcpy(m, macaddr, sizeof(macaddr));
      return 0;
	}
	return 1;
}
int CNetwork::GetMsgID()
{
   return txhdr.hdr.msgID;
}

void CNetwork::SetMsgID(int i)
{
   rxhdr.hdr.msgID=i;
   if(bBackoff){
		gtk_timeout_remove(timer_tag);
		bTimeout=false;
		bResend=false;
	}
}
	
int CNetwork::Send(BhPosHdr *hdr, bool bRs, bool bTo, int to_val)
{
	int ret;
	
	bTimeout=bTo;
	bResend=bRs;
	timer_value=to_val;
	
	retries=0;
   
	txhdr.hdr.type=hdr->hdr.type;
	txhdr.hdr.cmd=hdr->hdr.cmd;
	txhdr.hdr.subcmd=hdr->hdr.subcmd;
	txhdr.hdr.Ex=hdr->hdr.Ex;
	txhdr.hdr.size=hdr->hdr.size;
	
	// update once...
	if(hdr->hdr.type==BHPOS_TYPE_SYSTEM && hdr->hdr.cmd==BHMSG_SYSTEM_CONNECTION &&
	   hdr->hdr.subcmd==BHMSG_SYSTEM_CONNECT){
   	txhdr.hdr.sessionID=0;
   }
   
   txhdr.hdr.msgID=0;
	txhdr.hdr.timestamp++;
	txhdr.hdr.flags=hdr->hdr.flags;
	if(txhdr.hdr.size)
		memcpy(txhdr.data, hdr->data, txhdr.hdr.size);
	
#ifdef _DEBUGGERY
	g_message("\nMessage to - %s:%d:%d", inet_ntoa((struct in_addr) posserv.sin_addr), ntohs(posserv.sin_port), txhdr.hdr.sessionID);
	g_message("\nMessage len: %d,\n", sizeof(txhdr.hdr) + txhdr.hdr.size);
#endif

	ret=sendto(sd, &txhdr, sizeof(txhdr.hdr) + txhdr.hdr.size, 0, (struct sockaddr *) &posserv, sizeof(posserv));
	if(ret < 0){
		g_message("\nSend:: error=%d-%s", ret, strerror(errno));
	}
	
	if(bTimeout){
		timer_tag=gtk_timeout_add(timer_value, &toActive, this);
	}
	
	return ret;
}

int CNetwork::ReSend()
{
	int ret;
	
	txhdr.hdr.msgID++;

	ret=sendto(sd, &txhdr, sizeof(txhdr.hdr) + txhdr.hdr.size, 0, (struct sockaddr *) &posserv, sizeof(posserv));
	
	if(bTimeout){
		timer_tag=gtk_timeout_add(timer_value, &toActive, this);
	}
	return 0;
}
