/*
bhbookbox.cpp
Copyright (C) 2005 Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com  
*/

#include "config.h"
#include "bhintl.h"
#include "bhbookbox.h"

namespace bhpos{

namespace widget{

bhbookbox::bhbookbox(Glib::ustring& title, Glib::ustring& imagefile, Glib::ustring& message) 
{
   yes = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-find")));
   no = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-cancel")));
   add_action_widget(*yes, -6);
   add_action_widget(*no, -5);
   
   icon = manage(new class Gtk::Image(imagefile));
   l_msg = manage(new class Gtk::Label(message));
   l_isbn = manage(new class Gtk::Label(_("ISBN")));
   l_title = manage(new class Gtk::Label(_("Title")));
   l_author = manage(new class Gtk::Label(_("Author")));
      
   isbn = manage(new class Gtk::Entry());
   btitle = manage(new class Gtk::Entry());
   author = manage(new class Gtk::Entry());
   Gtk::HBox *hbox = manage(new class Gtk::HBox(false, 0));
   Gtk::VBox *vbox = Gtk::manage(new class Gtk::VBox(false, 0));
   Gtk::Table *table = Gtk::manage(new class Gtk::Table(3, 2, false));

   set_border_width(5);
   get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
   get_vbox()->set_homogeneous(false);
   get_vbox()->set_spacing(5);
   
   //icon->set_size(24, 24);
   icon->set_alignment(0.0, 0.5);
   icon->set_padding(10, 10);
   l_msg->set_alignment(0.5, 0.5);
   l_msg->set_padding(5, 5);
   l_isbn->set_alignment(1.0, 0.5);
   l_isbn->set_padding(5, 5);
   l_title->set_alignment(1.0, 0.5);
   l_title->set_padding(5, 5);
   l_author->set_alignment(1.0, 0.5);
   l_author->set_padding(5, 5);
   hbox->set_border_width(5);
   
   isbn->set_flags(Gtk::CAN_FOCUS);
   isbn->set_visibility(true);
   isbn->set_editable(true);
   isbn->set_max_length(0);
   isbn->set_text(_(""));
   isbn->set_has_frame(true);
   isbn->set_activates_default(true);
   
   btitle->set_flags(Gtk::CAN_FOCUS);
   btitle->set_visibility(true);
   btitle->set_editable(true);
   btitle->set_max_length(0);
   btitle->set_text(_(""));
   btitle->set_has_frame(true);
   
   author->set_flags(Gtk::CAN_FOCUS);
   author->set_visibility(true);
   author->set_editable(true);
   author->set_max_length(0);
   author->set_text(_(""));
   author->set_has_frame(true);
   
   table->attach(*l_isbn, 0,1,0,1, Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*isbn, 1,1,0,1, Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*l_title, 0,1,1,2,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*btitle, 1,1,1,2,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*l_author, 0,1,2,2,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*author, 1,1,2,2,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   
   vbox->pack_start(*l_msg, Gtk::PACK_SHRINK, 0);
   vbox->pack_start(*table, Gtk::PACK_SHRINK, 0);
   
   hbox->pack_start(*icon);
   hbox->pack_start(*vbox);
   get_vbox()->pack_start(*hbox);
   set_title(title);
   property_window_position().set_value(Gtk::WIN_POS_CENTER);
   set_resizable(false);
   property_destroy_with_parent().set_value(false);
   set_has_separator(true);
   
   set_modal(true);
   
   yes->show();
	no->show();
	
   isbn->grab_focus();
   show();
   yes->signal_clicked().connect(sigc::mem_fun(*this, &bhbookbox::on_yes_clicked));
   no->signal_clicked().connect(sigc::mem_fun(*this, &bhbookbox::on_no_clicked));
   isbn->signal_changed().connect(sigc::mem_fun(*this, &bhbookbox::on_isbn_changed), false);
}

void bhbookbox::on_isbn_changed()
{
   /*
   Glib::ustring a=amount->get_text();
   char buff[128];
   double d=atof(a.data());
   
   sprintf(buff, "%0.2f", d);
   
	event_signal.emit(1, atof(buff));
	this->destroy_();*/
}

void bhbookbox::Set(Glib::ustring& a, Glib::ustring& b, Glib::ustring& c)
{
   isbn->set_text(a);
   btitle->set_text(b);
   author->set_text(c);
}

void bhbookbox::on_yes_clicked()
{
   Glib::ustring i=isbn->get_text();
   Glib::ustring t=btitle->get_text();
   Glib::ustring a=author->get_text();
   
	event_signal.emit(i, t, a);
	this->destroy_();
}

void bhbookbox::on_no_clicked()
{
   Glib::ustring a="";
	event_signal.emit(a, a, a);
	this->destroy_();  
}

}

}
