/*
bhcustbox.cpp
Copyright (C) 2005 Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com  
*/

#include "config.h"
#include "bhintl.h"
#include "bhcustbox.h"

namespace bhpos{

namespace widget{

bhcustbox::bhcustbox(Glib::ustring& title, Glib::ustring& imagefile, Glib::ustring& message) 
{
   yes = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-find")));
   no = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-cancel")));
   add_action_widget(*yes, -6);
   add_action_widget(*no, -5);
   
   icon = manage(new class Gtk::Image(imagefile));
   l_msg = manage(new class Gtk::Label(message));
   l_title = manage(new class Gtk::Label(_("Title")));
   l_fname = manage(new class Gtk::Label(_("Forename")));
   l_lname = manage(new class Gtk::Label(_("Surname")));
   l_phone = manage(new class Gtk::Label(_("Phone Number")));
   l_pcode = manage(new class Gtk::Label(_("Post Code")));
      
   ctitle = manage(new class Gtk::Entry());
   fname = manage(new class Gtk::Entry());
   lname = manage(new class Gtk::Entry());
   phone = manage(new class Gtk::Entry());
   pcode = manage(new class Gtk::Entry());   
   
   Gtk::HBox *hbox = manage(new class Gtk::HBox(false, 0));
   Gtk::VBox *vbox = Gtk::manage(new class Gtk::VBox(false, 0));
   Gtk::Table *table = Gtk::manage(new class Gtk::Table(4, 2, false));

   set_border_width(5);
   get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
   get_vbox()->set_homogeneous(false);
   get_vbox()->set_spacing(5);
   
   //icon->set_size(24, 24);
   icon->set_alignment(0.0, 0.5);
   icon->set_padding(10, 10);
   
   l_msg->set_alignment(0.5, 0.5);
   l_msg->set_padding(5, 5);
   l_fname->set_alignment(1.0, 0.5);
   l_fname->set_padding(5, 5);
   l_lname->set_alignment(1.0, 0.5);
   l_lname->set_padding(5, 5);
   l_phone->set_alignment(1.0, 0.5);
   l_phone->set_padding(5, 5);
   l_pcode->set_alignment(1.0, 0.5);
   l_pcode->set_padding(5, 5);
   
   hbox->set_border_width(5);
   
   fname->set_flags(Gtk::CAN_FOCUS);
   fname->set_visibility(true);
   fname->set_editable(true);
   fname->set_max_length(0);
   fname->set_text(_(""));
   fname->set_has_frame(true);
   fname->set_activates_default(true);
   
   lname->set_flags(Gtk::CAN_FOCUS);
   lname->set_visibility(true);
   lname->set_editable(true);
   lname->set_max_length(0);
   lname->set_text(_(""));
   lname->set_has_frame(true);
   
   phone->set_flags(Gtk::CAN_FOCUS);
   phone->set_visibility(true);
   phone->set_editable(true);
   phone->set_max_length(0);
   phone->set_text(_(""));
   phone->set_has_frame(true);
   
   pcode->set_flags(Gtk::CAN_FOCUS);
   pcode->set_visibility(true);
   pcode->set_editable(true);
   pcode->set_max_length(0);
   pcode->set_text(_(""));
   pcode->set_has_frame(true);
   
   table->attach(*l_fname, 0,1,0,1, Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*fname, 1,1,0,1, Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*l_lname, 0,1,1,2,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*lname, 1,1,1,2,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*l_phone, 0,1,2,3,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*phone, 1,1,2,3,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*l_pcode, 0,1,3,3,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   table->attach(*pcode, 1,1,3,3,Gtk::SHRINK, Gtk::SHRINK, 0, 0);
   
   vbox->pack_start(*l_msg, Gtk::PACK_SHRINK, 0);
   vbox->pack_start(*table, Gtk::PACK_SHRINK, 0);
   
   hbox->pack_start(*icon);
   hbox->pack_start(*vbox);
   get_vbox()->pack_start(*hbox);
   set_title(title);
   property_window_position().set_value(Gtk::WIN_POS_CENTER);
   set_resizable(false);
   property_destroy_with_parent().set_value(false);
   set_has_separator(true);
   
   set_modal(true);
   
   yes->show();
	no->show();
	
   fname->grab_focus();
   show();
   yes->signal_clicked().connect(sigc::mem_fun(*this, &bhcustbox::on_yes_clicked));
   no->signal_clicked().connect(sigc::mem_fun(*this, &bhcustbox::on_no_clicked));
}

void bhcustbox::Set(Glib::ustring& a, Glib::ustring& b, Glib::ustring& c, Glib::ustring& d)
{
   fname->set_text(a);
   lname->set_text(b);
   phone->set_text(c);
   pcode->set_text(d);
}

void bhcustbox::on_yes_clicked()
{
   Glib::ustring a=fname->get_text();
   Glib::ustring b=lname->get_text();
   Glib::ustring c=phone->get_text();
   Glib::ustring d=pcode->get_text();
   
	event_signal.emit(a, b, c, d);
	this->destroy_();
}

void bhcustbox::on_no_clicked()
{
   Glib::ustring a="";
	event_signal.emit(a, a, a, a);
	this->destroy_();  
}

}

}
