/* bhentry.cpp
 *
 * Copyright (C) 2004 Al Kingsley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "bhentry.h"
#include <gtkmm/messagedialog.h>
#include <sstream> //For stringstream

#include <ctime>     // for struct tm
#include <iostream>   // for cout, endl

namespace bhpos{

namespace widget{

BhEntry::BhEntry(BaseObjectType* cobject) : Gtk::Entry(cobject),  type(TYPE_TEXT)
{
   mask="";
   format="";
   old_text="";
   lc=NULL;
   lc=(lconv *) malloc(sizeof(lconv));
	if(!lc){
		return;
	}
	lc=localeconv();
}
  
BhEntry::BhEntry(bhentry_type t) : type(t)
{
   mask="";
   format="";
   old_text="";
   lc=NULL;
   lc=(lconv *) malloc(sizeof(lconv));
	if(!lc){
		return;
	}
	lc=localeconv();
}

BhEntry::BhEntry()
{
   type=TYPE_NUMERIC;
   old_text="";
   mask="";
   format="";
   lc=NULL;
   lc=(lconv *) malloc(sizeof(lconv));
	if(!lc){
		return;
	}
	lc=localeconv();
}

BhEntry::~BhEntry()
{
   if(lc) free(lc);
}


/*
void SetActive(bool bActive)
{
   if(bActive){
   }
   else{
      set_visibility(true);
      set_sensitive(false);
      set_editable(false);
      set_max_length(0);
      set_text(_(""));
      set_has_frame(true);
      set_activates_default(false);
   }
}
*/
void BhEntry::SetColour(Gdk::Color col)
{
   modify_text(Gtk::STATE_INSENSITIVE, col);
}

void BhEntry::check_for_change()
{
   Glib::ustring::iterator iter;
   Glib::ustring valid="";
   Glib::ustring new_text = get_text();
   bool          success = true;
   select_region(0,0);
   set_position(new_text.size());
   
   if(new_text != old_text){
      switch(type){
      case TYPE_NONE:
         valid="";
         break;
      case TYPE_NUMERIC:
         valid="0123456789";
         break;
      case TYPE_CURRENCY:
         valid="0123456789.,";
         break;
      case TYPE_TEXT:
         valid="";
         break;
      case TYPE_DATE:
         valid="";
         break;
      case TYPE_TIME:
         valid="";
         break;
      case TYPE_DATETIME:
         valid="";
         break;
      case TYPE_BOOLEAN:
         new_text=new_text.lowercase();
         if(new_text=="true" || new_text=="false"){
            success=true;
         }
         break;
      }
      for(iter=new_text.begin(); iter!=new_text.end(); iter++){ 
         if(valid.find(*iter)==std::string::npos){
            success=false;
            break;
         }
      }
      /* this should work, but doesn't work:
      if(new_text.find_first_of(valid, 0)==std::string::npos && new_text.size()!=0){
         success=false;
      }
      */
      if(success){
         old_text=new_text;
         set_text(new_text);
         signal_edited.emit();
      }
      else{
         set_text(old_text);
         grab_focus();
      }
   }
}

  
BhEntry::type_signal_edited BhEntry::getsignaledited()
{
   return signal_edited;
}

bool BhEntry::on_focus_out_event(GdkEventFocus* event)
{
   bool result = Gtk::Entry::on_focus_out_event(event);
  
   //The user has finished editing.
   check_for_change();
  
   //Call base class:
   return result;
}

void BhEntry::on_activate()
{ 
   Gtk::Entry::on_activate();

   check_for_change();
}

void BhEntry::on_changed()
{
   Gtk::Entry::on_changed();
   //Glib::ustring new_text = get_text();
   check_for_change();
   //CheckText(new_text);
}

/*
bool BhEntry::CheckText(Glib::ustring new_text)
{
   
   int  pos=0;
   bool success = false;
   
   if(mask=="") return true;
   
   if(new_text==old_text) return true;
   
   switch(type){
   case TYPE_NONE:
      success=true;
      break;
      
   case TYPE_NUMERIC:
      // Processing of numeric masks:
      // #,9 -> Digit. If the first character is zero, it is replaced with a blank
      // 0 -> Digit. If the first character is zero, the zero is remained
      // . -> Decimal point position
      // , -> Thousands separator
      // + -> The number doesn't admit the negative sign
      // * -> Repeat the mask pattern
      for(Glib::ustring::iterator i=new_text.begin(); i!=new_text.end(); i++){
         switch(mask[pos]) {
         case '0':
            if(isdigit(new_text[pos])) break;
            else return false;
               
         case '#':
         case '9':
            if(isdigit(new_text[pos]) || new_text[pos]==' ') break;
            else return false;
               
         case ',':
            new_text.replace(i, i, nl_langinfo(THOUSANDS_SEP));
            break;
         
         case '.':
            new_text.replace(i, i, nl_langinfo(DECIMAL_POINT));
            break;
            
         default:
            //new_text.replace(i, i, mask[pos]);
            break;
         }
         pos++;
         if(pos==new_text.size()) return true;
      }
      break;
         
   case TYPE_CURRENCY:
      // check against format
      break;
   case TYPE_TEXT:
      // check against mask
      break;
   case TYPE_DATE:
      break;
   case TYPE_TIME:
      break;
   case TYPE_DATETIME:
      break;
   case TYPE_BOOLEAN:
      new_text=new_text.lowercase();
      if(new_text=="true" || new_text=="false"){
         success=true;
      }
      break;
   }
   return success;
}
*/
/*
void BhEntry::on_insert_text(const Glib::ustring& text, int* position)
{
   Gtk::Entry::on_insert_text(text, position);
}  

void BhEntry::set_text(const Glib::ustring& text)
{
   old_text = text;

   //Call base class:
   Gtk::Entry::set_text(text);
}
*/
}

}
