/*
bhitemlist.h
Copyright (C) 2004  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#ifndef _BHITEMLIST_H
#define _BHITEMLIST_H

#include <gdkmm/colormap.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treeview.h>
#include <gtkmm/treeviewcolumn.h>
#include <gtkmm/cellrenderertext.h>
#include <gtkmm/cellrenderercombo.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/entry.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/label.h>
#include <gtkmm/menu.h>
#include "bhcellrenderer.h"

#ifndef _BHHDR_H_
#include <pos.h>
#endif
#ifndef _CTRANSITEM_
#include <ctransitems.h>
#endif
#ifndef _CCASH_H_
#include <cash.h>
#endif

namespace bhpos{

/**
 * \ingroup mflib
 * @{	
 */
   
namespace widget{

/**
 * \defgroup bhitemlist Item List
 * @{
 *
 * \ingroup bhwidget
 * @{	
 */

enum BhItemListItemType{
   BHITEMLIST_TYPE_BLANK,
   BHITEMLIST_TYPE_ITEM,
   BHITEMLIST_TYPE_LOOKUP,
   BHITEMLIST_TYPE_UNKNOWN,
   BHITEMLIST_TYPE_PRICE,
   BHITEMLIST_TYPE_RETURN,
   BHITEMLIST_TYPE_DETAIL,
   BHITEMLIST_TYPE_DELIVERY,
   BHITEMLIST_TYPE_UNDELIVERED,
   BHITEMLIST_TYPE_LAYBY,
   BHITEMLIST_TYPE_RENTAL,
   BHITEMLIST_TYPE_RENTALRETURN,
   BHITEMLIST_TYPE_EPHEMERAL,
   BHITEMLIST_TYPE_TOTAL,
   BHITEMLIST_TYPE_TENDERTYPE_UNKNOWN,
   BHITEMLIST_TYPE_TENDERTYPE_CASH,
   BHITEMLIST_TYPE_TENDERTYPE_CREDIT,
	BHITEMLIST_TYPE_TENDERTYPE_CHEQUE,
	BHITEMLIST_TYPE_TENDERTYPE_SMART,
	BHITEMLIST_TYPE_TENDERTYPE_CREDITRETURN,
   BHITEMLIST_TYPE_TENDERTYPE_CREDITSLIP,
	BHITEMLIST_TYPE_TENDERTYPE_GIFTV,
	BHITEMLIST_TYPE_TENDERTYPE_EXTVOUCHER,
	BHITEMLIST_TYPE_TENDERTYPE_INTCOUPON,
   BHITEMLIST_TYPE_TENDERTYPE_EXTCOUPON,
	BHITEMLIST_TYPE_TENDERTYPE_LUNCHSTAMP,
   BHITEMLIST_TYPE_TENDERTYPE_INHOUSECHARGE,
   BHITEMLIST_TYPE_TENDERTYPE_STORECARD,
	BHITEMLIST_TYPE_TENDERTYPE_OTHER,
	BHITEMLIST_TYPE_TENDERTYPE_CREDITCB,
	BHITEMLIST_TYPE_TENDERTYPE_CHEQUECB,
	BHITEMLIST_TYPE_TENDERTYPE_SMARTCB,
	BHITEMLIST_TYPE_TENDERTYPE_TOKENCB,
	BHITEMLIST_TYPE_TENDERTYPE_VOUCHERCB,
	BHITEMLIST_TYPE_TENDERTYPE_COUPONCB,
	BHITEMLIST_TYPE_TENDERTYPE_OTHERCB
};

enum BhItemListItemStatus{
   BHITEMLIST_STATUS_NONE,
   BHITEMLIST_STATUS_INLOOKUP,
   BHITEMLIST_STATUS_ADD ,
   BHITEMLIST_STATUS_PLUEDIT,
   BHITEMLIST_STATUS_PLUSCAN,
   BHITEMLIST_STATUS_FIXED,
   BHITEMLIST_STATUS_QTYEDIT,
   BHITEMLIST_STATUS_DESCEDIT,
   BHITEMLIST_STATUS_PRICEEDIT,
   BHITEMLIST_STATUS_CASHITEM,
   BHITEMLIST_STATUS_DROPNEXT,
   BHITEMLIST_STATUS_AMOUNTEDIT,
   BHITEMLIST_STATUS_PAYFIXED
};

enum BhItemListItemFlag{
   BHITEMLIST_ITEMFLAG_NONE,
   BHITEMLIST_ITEMFLAG_NEW,
   BHITEMLIST_ITEMFLAG_PLUTEXT,
   BHITEMLIST_ITEMFLAG_PLUCLEAR,
   BHITEMLIST_ITEMFLAG_DROPNEXT,
   BHITEMLIST_ITEMFLAG_QTYNEW,
   BHITEMLIST_ITEMFLAG_QTYEDIT,
   BHITEMLIST_ITEMFLAG_PRICE,
   BHITEMLIST_ITEMFLAG_PRICEDESC,
   BHITEMLIST_ITEMFLAG_QTYINC,
   BHITEMLIST_ITEMFLAG_QTYDEC,
   BHITEMLIST_ITEMFLAG_NOCHECK
};

enum BhListActionType{
   BHITEMLIST_ACTION_NEW,
   BHITEMLIST_ACTION_KEY,
   BHITEMLIST_ACTION_SCANPLU,
   BHITEMLIST_ACTION_QTYNEW,
   BHITEMLIST_ACTION_QTYCHANGE,
   BHITEMLIST_ACTION_QTYMOVE,
   BHITEMLIST_ACTION_INCNEW,
   BHITEMLIST_ACTION_INCREMENT,
   BHITEMLIST_ACTION_INCMOVE,
   BHITEMLIST_ACTION_INCEDIT,
   BHITEMLIST_ACTION_DECREMENT,
   BHITEMLIST_ACTION_DECEDIT,
   BHITEMLIST_ACTION_PLULOOKUP,
   BHITEMLIST_ACTION_CANCELEDIT,
   BHITEMLIST_ACTION_SELPLU,
   BHITEMLIST_ACTION_PLURET,
   BHITEMLIST_ACTION_SELPRICE,
   BHITEMLIST_ACTION_CASHITEM,
   BHITEMLIST_ACTION_ITEMCLOSE,
   BHITEMLIST_ACTION_CLEAR,
   BHITEMLIST_ACTION_REMOVE,
   BHITEMLIST_ACTION_QTYSET,
   BHITEMLIST_ACTION_ADDTOLAST,
   BHITEMLIST_ACTION_UP,
   BHITEMLIST_ACTION_DOWN,
   BHITEMLIST_ACTION_PGUP,
   BHITEMLIST_ACTION_PGDOWN,
   BHITEMLIST_ACTION_HOME,
   BHITEMLIST_ACTION_END,
   BHITEMLIST_ACTION_PAYBALANCE,
   BHITEMLIST_ACTION_CLEARTENDERTYPE,
   BHITEMLIST_ACTION_CLEARPAYAMOUNT,
   BHITEMLIST_ACTION_CLEARALLPAYMENT
};

enum BhCompletionType{
   BHITEMLIST_COMPLETION_PLU,
   BHITEMLIST_COMPLETION_QTY,
   BHITEMLIST_COMPLETION_DESCEDIT,
   BHITEMLIST_COMPLETION_PRICEEDIT,
   BHITEMLIST_COMPLETION_INCEDIT,
   BHITEMLIST_COMPLETION_DECEDIT,
   BHITEMLIST_COMPLETION_REMOVE,
   BHITEMLIST_COMPLETION_CASHITEM,
   BHITEMLIST_COMPLETION_PAYAMOUNT,
   BHITEMLIST_COMPLETION_PAYBALANCE,
   BHITEMLIST_COMPLETION_PAYCHANGE,
   BHITEMLIST_COMPLETION_PAYAMOUNTMORE,
   BHITEMLIST_COMPLETION_PAYERROR,
   BHITEMLIST_COMPLETION_TENDER,
   BHITEMLIST_COMPLETION_PLUCANCELLED,
   BHITEMLIST_COMPLETION_QTYCANCELLED,
   BHITEMLIST_COMPLETION_DESCEDITCANCELLED,
   BHITEMLIST_COMPLETION_PRICEEDITCANCELLED,
   BHITEMLIST_COMPLETION_CASHITEMCANCELLED,
   BHITEMLIST_COMPLETION_PAYAMOUNTCANCELLED,
   BHITEMLIST_COMPLETION_PAYBALANCECANCELLED,
   BHITEMLIST_COMPLETION_TENDERCANCELLED,
};

enum BhItemListOption{
   BHITEMLIST_OPTION_TAXINCLUSIVE,
   BHITEMLIST_OPTION_PAYTYPES
   /*
   BHITEMLIST_OPTION_DEFAULT=0,
   BHITEMLIST_OPTION_EXPAND_TREE=1,
   BHITEMLIST_OPTION_ENABLE_MOUSE=2,
   BHITEMLIST_OPTION_ENABLE_POPUP=4*/
};

enum BhItemListMode{
   BHITEMLIST_MODE_IDLE,
   BHITEMLIST_MODE_PRICELIST,
   BHITEMLIST_MODE_SALE,
   BHITEMLIST_MODE_QTYEDIT,
   BHITEMLIST_MODE_WAIT, 
   BHITEMLIST_MODE_TENDERTYPE,
   BHITEMLIST_MODE_TENDERAMOUNT,
   BHITEMLIST_MODE_TRANSLIST,
   BHITEMLIST_MODE_EDITSALE,
   BHITEMLIST_MODE_EDITPAY,
   BHITEMLIST_MODE_PRICEADD,
   BHITEMLIST_MODE_PAYWAIT,
   BHITEMLIST_MODE_PAYENTRY
};

enum BhNavigationType{
   BHNAV_TYPE_UP,
   BHNAV_TYPE_DOWN,
   BHNAV_TYPE_PGUP,
   BHNAV_TYPE_PGDOWN,
   BHNAV_TYPE_HOME,
   BHNAV_TYPE_END,
   BHNAV_TYPE_LEFT,
   BHNAV_TYPE_RIGHT
};
/*
enum BhItemListSignal{
   BHITEMLIST_SIG_PLUENTRY,
   BHITEMLIST_SIG_PLUDONE,
   BHITEMLIST_SIG_EDIT
};
*/
#define BHCOL_ROW_NONE_BACKGROUND         1
#define BHCOL_ROW_NONE_FOREGROUND         2
#define BHCOL_ROW_ITEM_BACKGROUND         3
#define BHCOL_ROW_ITEM_FOREGROUND         4
#define BHCOL_ROW_ITEM_BACKGROUND_SEL     5
#define BHCOL_ROW_ITEM_FOREGROUND_SEL     6
#define BHCOL_ROW_UNKNOWN_BACKGROUND      7
#define BHCOL_ROW_UNKNOWN_FOREGROUND      8
#define BHCOL_ROW_UNKNOWN_BACKGROUND_SEL  9
#define BHCOL_ROW_UNKNOWN_FOREGROUND_SEL  10
#define BHCOL_ROW_LOOKUP_BACKGROUND       11
#define BHCOL_ROW_LOOKUP_FOREGROUND       12
#define BHCOL_ROW_PRICE_BACKGROUND        13
#define BHCOL_ROW_PRICE_FOREGROUND        14
#define BHCOL_ROW_RETURN_BACKGROUND       15
#define BHCOL_ROW_RETURN_FOREGROUND       16
#define BHCOL_ROW_LAYBY_BACKGROUND        17
#define BHCOL_ROW_LAYBY_FOREGROUND        18
#define BHCOL_ROW_RENTAL_BACKGROUND       19
#define BHCOL_ROW_RENTAL_FOREGROUND       20
#define BHCOL_ROW_EPHEMERAL_BACKGROUND    21
#define BHCOL_ROW_EPHEMERAL_FOREGROUND    22
#define BHCOL_ROW_EDIT_BACKGROUND         23
#define BHCOL_ROW_EDIT_FOREGROUND         24
#define BHCOL_ROW_EDITUNKNOWN_BACKGROUND  25
#define BHCOL_ROW_EDITUNKNOWN_FOREGROUND      26
#define BHCOL_ROW_EDITUNKNOWN_BACKGROUND_SEL  27
#define BHCOL_ROW_EDITUNKNOWN_FOREGROUND_SEL  28
#define BHCOL_ROW_TOTAL_BACKGROUND            29
#define BHCOL_ROW_TOTAL_FOREGROUND            30
#define BHCOL_ROW_TENDER_BACKGROUND           31
#define BHCOL_ROW_TENDER_FOREGROUND           32
#define BHCOL_ROW_TENDEREDIT_BACKGROUND       33
#define BHCOL_ROW_TENDEREDIT_FOREGROUND       34

/*! SaleColumns
 \brief A class to handle item columns
*/
class SaleColumns : public Gtk::TreeModel::ColumnRecord
{
public:
   SaleColumns();
   Gtk::TreeModelColumn<Glib::ustring> plu;
   Gtk::TreeModelColumn<Glib::ustring> quantity;
   Gtk::TreeModelColumn<Glib::ustring> description;
   Gtk::TreeModelColumn<Glib::ustring> subtotal;
   Gtk::TreeModelColumn<Glib::ustring> total;
   Gtk::TreeModelColumn<int> 	         status;	   // hidden col
   Gtk::TreeModelColumn<int>           type;	      // hidden col
   Gtk::TreeModelColumn<Gdk::Color>    background; // hidden col
   Gtk::TreeModelColumn<Gdk::Color>    foreground; // hidden col
};

/*! SaleColours
 \brief A class to handle item list colours
*/
class SaleColours
{
protected:
   Glib::RefPtr<Gdk::Colormap> colormap;
   Gdk::Color row_item_foreground;
   Gdk::Color row_item_background;
   Gdk::Color row_item_foreground_sel;
   Gdk::Color row_item_background_sel;
   Gdk::Color row_unknown_foreground;
   Gdk::Color row_unknown_background;
   Gdk::Color row_unknown_foreground_sel;
   Gdk::Color row_unknown_background_sel;
   Gdk::Color row_lookup_foreground;
   Gdk::Color row_lookup_background;
   Gdk::Color row_return_foreground;
   Gdk::Color row_return_background;
   Gdk::Color row_layby_foreground;
   Gdk::Color row_layby_background;
   Gdk::Color row_rental_foreground;
   Gdk::Color row_rental_background;
   Gdk::Color row_ephemeral_foreground;
   Gdk::Color row_ephemeral_background;
   Gdk::Color row_priceitem_foreground;
   Gdk::Color row_priceitem_background;
   Gdk::Color row_edit_foreground;
   Gdk::Color row_edit_background;
   Gdk::Color row_editunknown_foreground;
   Gdk::Color row_editunknown_background;
   Gdk::Color row_editunknown_foreground_sel;
   Gdk::Color row_editunknown_background_sel;
   Gdk::Color row_total_foreground;
   Gdk::Color row_total_background;
   Gdk::Color row_tender_foreground;
   Gdk::Color row_tender_background;
   Gdk::Color row_tenderedit_foreground;
   Gdk::Color row_tenderedit_background;

public:
   SaleColours(Gtk::Widget *);
   void Set(int colour, int r, int g, int b);
   Gdk::Color Get(int colour);
};

/** bhitemlist class 
* \brief GUI implementation of the gkmm treeview widget.
* A sale items list.
*
* TreeStore was used for multiple sale item columns (at present: discount,
* deposit, and flags - will be extended later). 
*
* This version is also mouse enabled, unlike previous versions, but this 
* can be disabled by setting the option. The reason for this was touchscreen
* interfacing.
*/ 
class BhItemList : public Gtk::TreeView
{
private:
   Glib::ustring buffer;
   Gtk::TreeModel::iterator curr_row;
   int           i_curr_row;
   int           row_height;
   int           num_rows;
   int           curr_column;
   bool          bDropNext;
   bool          bHaveRow;
   bool          bTaxInclusive;
   bool          bShortList;
   double        due;
   double        total;
   BhItemListMode   mode;

   CPayItem      curr_payitem;
   
   Gtk::Menu     item_popup;
   Gtk::Menu     pay_popup;

   Gtk::Entry             *cellEntry;
   //BhCellRendererText *cell_col0;
   Gtk::CellRendererText  *cell_col0;
   Gtk::CellRendererText  *cell_col1;
   Gtk::CellRendererText  *cell_col2;
   Gtk::CellRendererText  *cell_col3;
   Gtk::CellRendererText  *cell_col4;
   
   bool                    bCell0Retry;
   Glib::ustring           cell0_oldtext;
   Gtk::CellRendererCombo *cell_col3pay;
   Gtk::TreeView::Column   col0_validated;

   Gtk::TreeModel::Row  total_row;
      
   SaleColours *colours;

   void SetSelection(bool bMultiple);
   void Select(int n);
   //void text_row_func(Gtk::CellRenderer* cell, const Gtk::TreeModel::iterator& iter);
   void on_col0_changed(const Glib::ustring& path, const Glib::ustring& new_text);
   void on_col1_changed(const Glib::ustring& path, const Glib::ustring& new_text);
   void on_col2_changed(const Glib::ustring& path, const Glib::ustring& new_text);
   void on_col3_changed(const Glib::ustring& path, const Glib::ustring& new_text);
   void on_col4_changed(const Glib::ustring& path, const Glib::ustring& new_text);
   
   void on_col0_validate(Gtk::CellRenderer *renderer, const Gtk::TreeModel::iterator& iter);
   void cell0_editstarted(Gtk::CellEditable *cell_editable, const Glib::ustring& path);
   void cell0_editcancelled();
   void cell1_editcancelled();
   void cell2_editcancelled();
   void cell3_editcancelled();
   
   bool IsValidPath(Gtk::TreeModel::Path path);
   bool fillitem(CSaleItem *si);
   bool fillitem(CPayItem *pi);
   
   bool is_last_row(Gtk::TreeModel::iterator& iter_);
   void scroll(bool bUp);
   
   bool FindRow(Gtk::TreeModel::Row& row, Glib::ustring plu);
   bool FindRow(Gtk::TreeModel::Row& row, CSaleItem& si);
   bool FindRow(Gtk::TreeModel::Row& row, CPayItem& si);
   BhItemListItemType PayType(BhPayType t);

protected:
	SaleColumns                      columns;
   Glib::RefPtr<Gtk::TreeStore>     refls;
   Glib::RefPtr<Gtk::TreeSelection> refSelection;
   Gtk::ScrolledWindow              *scroll_win;
/*
   void on_selection_changed();
   bool select_function(const Glib::RefPtr<Gtk::TreeModel>& model,const Gtk::TreeModel::Path& path, bool);
   bool on_button_press_event(GdkEventButton *ev);*/
   void on_sale_popup();
   void on_pay_popup();
   
   int Update(CTransItems& transaction);
   int SaleMode(CTransItems& transaction);
   
   void SetRowColour(Gtk::TreeModel::iterator iter);
   void SetRowColour(Gtk::TreeModel::Row& row);
   

   //int TransListMode(BhTransList *tl, bool bShow);
   /*
   int TransSummary(bool, BhServiceTransSum *ts);
   int UserSummary(bool, BhServiceUserSum *us);
   int CashDrawerAdjust(CCash& cashd, bool bShow);
   int EphemeralsIn(bool bShow);
   int Ephemerals(CTransItems&, CTransItems&, bool bShow);
   int CustomerList(BhCustomerList *cl, bool bShow);
   int CustomerSelection(char *);
   int DeliveryList(BhDeliveryList *, bool bShow);
   int DeliverySummary(BhDeliveryList *dl, bool bShow);
   int DeliveryReturnList(BhDeliveryReturnList *, bool bShow);
   int DeliveryPay(BhDeliveryReturnList *, bool bShow);
   int RentalList(BhRentalList *, bool bShow);
   int RentalReturn(BhRentalList *, bool bShow);
   int Layby(CTransItems&, bool bShow);
   int LaybySummary(BhLaybyList *, bool bShow);
   int LaybyList(BhLaybyList *, bool bShow);
   int LaybyPay(BhLaybyList *, bool bShow);
   int ReturnReason(BhReturnReasonList *rl, bool bShow);
   int OpenTransList(BhTransRoverList *rl, bool bShow);
   int OpeningBalance(CCash& cashd, bool bShow);
   int BlindBalance(BhBlindBalanceReply *, bool bShow);
*/
   //sigc::signal<int, int, CSaleitem *> sitemchanged_signal;
   //sigc::signal<int, int, CPayitem *>  pitemchanged_signal;
   sigc::signal<void, int, Glib::ustring&> pluentry_signal;
   sigc::signal<void, BhCompletionType, void *, void *> completion_signal;

public:
   BhItemList(Gtk::ScrolledWindow *sw);
   ~BhItemList();
   void SetOption(BhItemListOption, void *);
   void Resize();
   /** Load a transaction 
    *  <tt>Clears current items, loads new transaction</tt>
    *  \param transaction to be filled.
    *  \return 1 is trans has 0 items, or bad load, 1 otherwise.
    */
   int  Load(CTransItems& transaction);
   int  PriceList(BhPriceList *pl, bool bShow);
   int  TransList(BhTransList *tl, bool bShow);
   int  TransList(BhSSTransList *tl, bool bShow);
   void Clear();
   int PayMode(double, bool bShow);
   void SetMode(BhItemListMode, void *);
   BhItemListMode GetMode();
   //bool Enter(Glib::ustring& edited_value);
   //bool Get(BhItemListItemFlag flag, int row, int column, CSaleItem& si);
   bool Get(double *);
   bool Get(int *qty);

   /** Add
    *  \brief Add an item to the list
    */
   int  Add(BhItemListItemFlag flag, CSaleItem *saleitem);
   void Append(CSaleItem *saleitem);
   int  Add(CPayItem& payitem);
   int  AddPayType(BhPayType paytype);
   
   /** Edit
    *  \brief Edit item in the list
    */
   int  Edit(CSaleItem& oldsaleitem, CSaleItem& newsaleitem);
   int  Edit(CPayItem& oldpayitem, CPayItem& newpayitem);
   int  Edit(int row, CSaleItem& saleitem);
   int  Edit(int row, CPayItem& payitem);
   int  Edit(CSaleItem& saleitem);
   int  Edit(CPayItem& payitem);
   
   /** Remove
    *  \brief Remove an item from the list
    */
   int  Remove(CSaleItem& saleitem);
   int  Remove(CPayItem& payitem);
   int  Remove(int row);
   
   /** General Action
    *  \brief A general action to perform
    *  \param type Action type.
    *  \return true if action failed, false otherwise.
    */
   bool Action(BhListActionType type, void *item);
   bool Navigation(BhNavigationType n);
   
   sigc::signal<void, int, Glib::ustring&> signal_pluentry(){ return pluentry_signal; }
   sigc::signal<void, BhCompletionType, void *, void *> signal_completion(){ return completion_signal; }
   //sigc::signal<int, int, CSaleitem *> signal_saleitem_changed(){ return sitemchanged_signal; }
   //sigc::signal<int, int, CPayitem *> signal_payitem_changed(){ return pitemchanged_signal; }
};

/* @} 
	@} */

}

/* @} */

}

#endif   // _BHITEMLIST_H
