/*
bhpaybox.cpp
Copyright (C) 2005 Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/

#include "config.h"
#include "bhintl.h"
#include "bhpaybox.h"

namespace bhpos{

namespace widget{

bhpaybox::bhpaybox(BhPayBoxType type, Glib::ustring& title, Glib::ustring& pm, Glib::ustring& message) 
{
   yes = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-ok")));
   no = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-cancel")));
   add_action_widget(*yes, -6);
   add_action_widget(*no, -5);
      
   switch(type){
   case BHPAYBOX_IN:
   case BHPAYBOX_OUT:
   case BHPAYBOX_ADJUSTMENT:
      
   	break;
   }
   
   icon = manage(new class Gtk::Image(pm));
   label = manage(new class Gtk::Label(message));
   amount = manage(new class Gtk::Entry());
   Gtk::HBox *hbox = manage(new class Gtk::HBox(false, 0));
   Gtk::VBox *vbox = Gtk::manage(new class Gtk::VBox(false, 0));

   set_border_width(5);
   get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
   get_vbox()->set_homogeneous(false);
   get_vbox()->set_spacing(5);
   
   //icon->set_size(24, 24);
   icon->set_alignment(0.0, 0.5);
   icon->set_padding(10, 10);
   label->set_alignment(1.0, 0.5);
   label->set_padding(5, 5);
   hbox->set_border_width(5);
   
   amount->set_flags(Gtk::CAN_FOCUS);
   amount->set_visibility(true);
   amount->set_editable(true);
   amount->set_max_length(0);
   amount->set_text(_(""));
   amount->set_has_frame(true);
   amount->set_activates_default(true);
   
   vbox->pack_start(*label, Gtk::PACK_SHRINK, 0);
   vbox->pack_start(*amount, Gtk::PACK_SHRINK, 0);
   
   hbox->pack_start(*icon);
   hbox->pack_start(*vbox);
   get_vbox()->pack_start(*hbox);
   set_title(title);
   property_window_position().set_value(Gtk::WIN_POS_CENTER);
   set_resizable(false);
   property_destroy_with_parent().set_value(false);
   set_has_separator(true);
   
   set_modal(true);
   
   yes->show();
	no->show();
	
   icon->show();
   label->show();
   amount->show();
   vbox->show();
   hbox->show();
   amount->grab_focus();
   show();
   yes->signal_clicked().connect(sigc::mem_fun(*this, &bhpaybox::on_yes_clicked));
   no->signal_clicked().connect(sigc::mem_fun(*this, &bhpaybox::on_no_clicked));
   amount->signal_changed().connect(sigc::mem_fun(*this, &bhpaybox::on_amount_changed), false);
}

void bhpaybox::on_amount_changed()
{
   /*
   Glib::ustring a=amount->get_text();
   char buff[128];
   double d=atof(a.data());
   
   sprintf(buff, "%0.2f", d);
   
	event_signal.emit(1, atof(buff));
	this->destroy_();*/
}

void bhpaybox::on_yes_clicked()
{
   Glib::ustring a=amount->get_text();
   char buff[128];
   double d=atof(a.data());
   
   sprintf(buff, "%0.2f", d);
   
	event_signal.emit(1, atof(buff));
	this->destroy_();
}

void bhpaybox::on_no_clicked()
{
	event_signal.emit(0, 0.0);
	this->destroy_();  
}

}

}
