/* bhstatus.cpp
 *
 * Copyright (C) 2004 Al Kingsley
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "config.h"
#include "bhstatus.h"
#include "bhintl.h"

namespace bhpos{

namespace widget{

BhStatus::BhStatus(BaseObjectType* cobject) : Gtk::HBox(cobject)
{
   Gdk::Color col;
   
   status = Gtk::manage(new class Gtk::Entry());
   user = Gtk::manage(new class Gtk::Entry());
   mode = Gtk::manage(new class Gtk::Entry());
   system = Gtk::manage(new class Gtk::Entry());
    
   status->set_visibility(true);
   status->set_editable(false);
   status->set_max_length(0);
   status->set_text(_(""));
   status->set_has_frame(true);
   status->set_activates_default(false);
   status->property_can_focus ()=false;
   
   user->set_visibility(true);
   user->set_editable(false);
   user->set_max_length(0);
   user->set_text(_(""));
   user->set_has_frame(true);
   user->set_activates_default(false);
   user->property_can_focus ()=false;
   
   mode->set_visibility(true);
   mode->set_editable(false);
   mode->set_max_length(0);
   mode->set_text(_(""));
   mode->set_has_frame(true);
   mode->set_activates_default(false);
   mode->property_can_focus()=false;
   mode->property_sensitive()=false;
   
   system->set_visibility(true);
   system->set_editable(false);
   system->set_max_length(0);
   system->set_text(_(""));
   system->set_has_frame(true);
   system->set_activates_default(false);
   system->property_can_focus()=false;
   system->property_sensitive()=false;

   col.set_rgb(65535, 65535, 65535);
   status->modify_bg(Gtk::STATE_INSENSITIVE, col);
   status->modify_bg(Gtk::STATE_SELECTED, col);
   status->modify_bg(Gtk::STATE_PRELIGHT, col);
   status->modify_bg(Gtk::STATE_ACTIVE, col);
   status->modify_bg(Gtk::STATE_NORMAL, col);
   
   status->modify_base(Gtk::STATE_INSENSITIVE, col);
   status->modify_base(Gtk::STATE_SELECTED, col);
   status->modify_base(Gtk::STATE_PRELIGHT, col);
   status->modify_base(Gtk::STATE_ACTIVE, col);
   status->modify_base(Gtk::STATE_NORMAL, col);
      
   user->modify_bg(Gtk::STATE_INSENSITIVE, col);
   user->modify_base(Gtk::STATE_INSENSITIVE, col);
   mode->modify_bg(Gtk::STATE_INSENSITIVE, col);
   mode->modify_base(Gtk::STATE_INSENSITIVE, col);
   system->modify_bg(Gtk::STATE_INSENSITIVE, col);
   system->modify_base(Gtk::STATE_INSENSITIVE, col);
   
   col.set_rgb(0, 0, 0);
   status->modify_text(Gtk::STATE_INSENSITIVE, col);
   user->modify_text(Gtk::STATE_INSENSITIVE, col);
   mode->modify_text(Gtk::STATE_INSENSITIVE, col);

   col.set_rgb(8704, 31744, 12288);
   system->modify_text(Gtk::STATE_INSENSITIVE, col);
   
   pack_start(*status, Gtk::PACK_SHRINK, 0);
   pack_start(*user, Gtk::PACK_SHRINK, 0);
   pack_start(*mode, Gtk::PACK_SHRINK, 0);
   pack_start(*system, Gtk::PACK_SHRINK, 0);
}

BhStatus::BhStatus() : Gtk::HBox(true, 10)
{
   Gdk::Color col;
   
   status = Gtk::manage(new class Gtk::Entry());
   user = Gtk::manage(new class Gtk::Entry());
   mode = Gtk::manage(new class Gtk::Entry());
   system = Gtk::manage(new class Gtk::Entry());
    
   status->set_visibility(true);
   status->set_editable(false);
   //status->set_max_length(0);
   status->set_text(_(""));
   status->set_has_frame(true);
   status->set_activates_default(false);
   status->property_can_focus ()=false;
   status->property_sensitive()=false;
   
   user->set_visibility(true);
   user->set_editable(false);
   user->set_max_length(0);
   user->set_text(_(""));
   user->set_has_frame(true);
   user->set_activates_default(false);
   user->property_can_focus ()=false;
   user->property_sensitive()=false;
   
   mode->set_visibility(true);
   mode->set_editable(false);
   mode->set_max_length(0);
   mode->set_text(_(""));
   mode->set_has_frame(true);
   mode->set_activates_default(false);
   mode->property_can_focus()=false;
   mode->property_sensitive()=false;
   
   system->set_visibility(true);
   system->set_editable(false);
   system->set_max_length(0);
   system->set_text(_(""));
   system->set_has_frame(true);
   system->set_activates_default(false);
   system->property_can_focus()=false;
   system->property_sensitive()=false;
   
   col.set_rgb(65535, 65535, 65535);
   status->modify_bg(Gtk::STATE_INSENSITIVE, col);
   status->modify_base(Gtk::STATE_INSENSITIVE, col);
   user->modify_bg(Gtk::STATE_INSENSITIVE, col);
   user->modify_base(Gtk::STATE_INSENSITIVE, col);
   mode->modify_bg(Gtk::STATE_INSENSITIVE, col);
   mode->modify_base(Gtk::STATE_INSENSITIVE, col);
   system->modify_bg(Gtk::STATE_INSENSITIVE, col);
   system->modify_base(Gtk::STATE_INSENSITIVE, col);
   
   pack_start(*status, Gtk::PACK_SHRINK, 0);
   pack_start(*user, Gtk::PACK_SHRINK, 0);
   pack_start(*mode, Gtk::PACK_SHRINK, 0);
   pack_start(*system, Gtk::PACK_SHRINK, 0);
}
  
BhStatus::~BhStatus()
{
}

void BhStatus::SetText(BhStatusItem b, Glib::ustring buff)
{
	switch(b){
	case BHSTATUS_STATUS:
		status->set_text(buff);
		break;
		
	case BHSTATUS_USER:
		user->set_text(buff);
		break;
	
	case BHSTATUS_MODE:
		mode->set_text(buff);
		break;
		
	case BHSTATUS_SYSTEM:
		system->set_text(buff);
		break;
	}
}

void BhStatus::SetColour(BhStatusItem i, Gtk::StateType t, Gdk::Color col)
{
   switch(i){
	case BHSTATUS_STATUS:
		status->modify_text(t, col);
		break;
		
	case BHSTATUS_USER:
		user->modify_text(t, col);
		break;
	
	case BHSTATUS_MODE:
		mode->modify_text(t, col);
		break;
		
	case BHSTATUS_SYSTEM:
		system->modify_text(t, col);
		break;
	}
}

void BhStatus::SetBackground(BhStatusItem i, Gtk::StateType t, Gdk::Color col)
{
   switch(i){
	case BHSTATUS_STATUS:
      status->modify_base(t, col);
		status->modify_bg(t, col);
		break;
		
	case BHSTATUS_USER:
      user->modify_base(t, col);
		user->modify_bg(t, col);
		break;
	
	case BHSTATUS_MODE:
      mode->modify_base(t, col);
		mode->modify_bg(t, col);
		break;
		
	case BHSTATUS_SYSTEM:
      system->modify_base(t, col);
		system->modify_bg(t, col);
		break;
	}
}

}

}
